<?php
namespace App\Services\Script;

use App\Services\Script\IScriptService;
use Carbon\Carbon;
use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Cache;
use App\Facades\Util\Log;
use App\Models\WarnConfigModel;
use App\Models\LastIdModel;
use App\Models\WarnWaitModel;
use App\Models\UserCashModel;
use App\Models\SyceeWithdrawModel;
use App\Models\WarnPeopleModel;

class ScriptService implements IScriptService
{
    private $time;

    public function __construct()
    {
        $this->time = Carbon::now()->format('Y-m-d H:i:s');
    }

    /**
     * 单次提现最高预警
     * @return int
     * @date 2021/08/02
     * @author live
     */
    public function getSingleWithdrawal()
    {
        $config = self::getWarnConfig(1);
        $oneMin = $config['oneMin'] ?? 0;
        if(empty($config) || $oneMin == 0) return true;
        $lastId = self::getLastDataId(2);

        $warnPeople = self::getWarnPeople();
        $logs = SyceeWithdrawModel::where('sycee_withdraw.serial','>',$lastId)->where('sycee_withdraw.cash','>',$oneMin)->join('account','sycee_withdraw.user_id','account.user_id')->orderBy('sycee_withdraw.serial','ASC')->select('account.user_id','account.platform','account.platform_id','account.channel','sycee_withdraw.serial','sycee_withdraw.cash','sycee_withdraw.withdraw_time')->limit(100)->get();

        $array = [];
        $maxId = 0;
        foreach($logs as $log){
            $maxId = $log->serial;
            $key = $log->platform.'-'.$log->channel;
            $warnConfig = $config[$key] ?? [];
            if(empty($warnConfig)) continue;
            $people = $warnPeople[$key] ?? [];
            if(empty($people)) continue;

            $tem['ymd'] = date('Ymd',strtotime($log->withdraw_time));
            $tem['config_id'] = $warnConfig['id'];
            $tem['type'] = 1;
            $tem['cash'] = $log->cash;
            $tem['warn_value'] = $warnConfig['one'];
            $tem['people_id'] = implode('#',$people['id']);
            $tem['people_phone'] = implode('#',$people['phone']);
            $tem['people_name'] = implode('#',$people['name']);
            $tem['user_id'] = $log->platform_id;
            $tem['open_id'] = $log->user_id;
            $tem['status'] = 0;
            $tem['created_at'] = $this->time;
            $tem['updated_at'] = $this->time;
            $array[] = $tem;
        }
        DB::beginTransaction();
        try{
            if(!empty($array)) WarnWaitModel::insert($array);
            if($maxId > 0) self::getUpdateLastId(2,$maxId);
            DB::commit();
            return true;
        } catch (\Exception $e){
            DB::rollback();
            return false;
        }
    }


    /**
     * 获取预警人信息
     * @return array
     * @date 2021/08/02
     * @author live
     */
    public static function getWarnPeople()
    {
        $array = [];
        $peoples =  WarnPeopleModel::where('status',1)->select('id','platform','channel','name','phone')->get();
        foreach($peoples as $peo){
            $key = $peo->platform.'-'.$peo->channel;

            if(isset($array[$key])){
                $array[$key]['id'][] = $peo->id;
                $array[$key]['name'][] = $peo->name;
                $array[$key]['phone'][] = $peo->phone;
            }else{
                $tem['id'][] = $peo->id;
                $tem['name'][] = $peo->name;
                $tem['phone'][] = $peo->phone;
                $array[$key] = $tem;
            }
        }
        unset($peoples,$peo,$tem);
        return $array;
    }

    /**
     * 跑完脚本更新最后一组数据ID
     * @return array
     * @date 2021/08/02
     * @author live
     */
    public static function getUpdateLastId($type,$maxId)
    {
        $last = LastIdModel::where('type',$type)->select('id','last_id')->first();
        if($last){
            $last->last_id = $maxId;
        }else{
            $last = new LastIdModel();
            $last->type = $type;
            $last->last_id = $maxId;
            $last->created_at = Carbon::now()->format('Y-m-d H:i:s');
        }
        $last->save();
        return true;
    }

    /**
     * 获取最后一跑数据的ID
     * @return array
     * @date 2021/08/02
     * @author live
     */
    public static function getLastDataId($type)
    {
        $last = LastIdModel::where('type',$type)->select('id','last_id')->first();
        return $last->last_id ?? 0;
    }

    /**
     * 获取预警配置
     * @return array
     * @date 2021/08/02
     * @author live
     */
    public static function getWarnConfig($type)
    {
        $array = ['oneMin'=>0,'dayMin'=>0,'userMin'=>0];
        $configs = WarnConfigModel::where('status',1)->select('id','platform','channel','one_withdrawal','day_withdrawal','people_day_withdrawal')->get();

        if(!$configs->count()) return [];
        foreach($configs as $con){
            $key = $con->platform.'-'.$con->channel;

            if($array['oneMin'] == 0 || $array['oneMin'] > $con->one_withdrawal) $array['oneMin'] = $con->one_withdrawal;
            if($array['dayMin'] == 0 || $array['dayMin'] > $con->day_withdrawal) $array['dayMin'] = $con->day_withdrawal;
            if($array['userMin'] == 0 || $array['userMin'] > $con->people_day_withdrawal) $array['userMin'] = $con->people_day_withdrawal;

            if(isset($array[$key])){
                if($array[$key]['one'] > $con->one_withdrawal){
                    $array[$key]['one'] = $con->one_withdrawal;
                    if($type == 1) $array[$key]['id'] = $con->id;
                }
                if($array[$key]['day'] > $con->day_withdrawal){
                    $array[$key]['day'] = $con->day_withdrawal;
                    if($type == 2) $array[$key]['id'] = $con->id;
                }
                if($array[$key]['user'] > $con->people_day_withdrawal){
                    $array[$key]['user'] = $con->people_day_withdrawal;
                    if($type == 3) $array[$key]['id'] = $con->id;
                }
            }else{
                $tem['id'] = $con->id;
                $tem['one'] = $con->one_withdrawal;
                $tem['day'] = $con->day_withdrawal;
                $tem['user'] = $con->people_day_withdrawal;
                $array[$key] = $tem;
            }
        }

        unset($configs,$con,$tem);
        return $array;
    }
}